@@ -12,7 +12,7 @@
 #   Craig Barratt  <cbarratt@users.sourceforge.net>
 #
 # COPYRIGHT
-#   Copyright (C) 2004-2019  Craig Barratt
+#   Copyright (C) 2004-2020  Craig Barratt
 #
 #   This program is free software: you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -29,7 +29,7 @@
 #
 #========================================================================
 #
-# Version 4.3.1, released 14 Jul 2019.
+# Version 4.4.0, released 20 Jun 2020.
 #
 # See http://backuppc.sourceforge.net.
 #
@@ -49,10 +49,7 @@ sub new
     my $class = shift;
     my($flds, $paths) = @_;
 
-    my $s = bless {
-	%$flds,
-	%$paths,
-    }, $class;
+    my $s = bless {%$flds, %$paths}, $class;
     return $s;
 }
 
@@ -76,7 +73,7 @@ sub BackupInfoRead
         $locked = 1;
     }
     if ( open($bkFd, "$s->{TopDir}/pc/$host/backups") ) {
-	binmode($bkFd);
+        binmode($bkFd);
         while ( <$bkFd> ) {
             s/[\n\r]+//;
             next if ( !/^(\d+\t(incr|full|partial|active).*)/ );
@@ -89,17 +86,30 @@ sub BackupInfoRead
         flock($lockFd, LOCK_UN);
         close($lockFd);
     }
-    #
-    # Default the version field.  Prior to 3.0.0 the xferMethod
-    # field is empty, so we use that to figure out the version.
-    #
     for ( my $i = 0 ; $i < @Backups ; $i++ ) {
-        next if ( $Backups[$i]{version} ne "" );
-        if ( $Backups[$i]{xferMethod} eq "" ) {
-            $Backups[$i]{version} = "2.1.2";
-        } else {
-            $Backups[$i]{version} = "3.0.0";
+        #
+        # Default the version field.  Prior to 3.0.0 the xferMethod
+        # field is empty, so we use that to figure out the version.
+        #
+        if ( $Backups[$i]{version} eq "" ) {
+            if ( $Backups[$i]{xferMethod} eq "" ) {
+                $Backups[$i]{version} = "2.1.2";
+            } else {
+                $Backups[$i]{version} = "3.0.0";
+            }
         }
+        #
+        # Reconstitue share2path into a hash
+        #
+        if ( $Backups[$i]{share2path} ne "" ) {
+            my $str = $Backups[$i]{share2path};
+            $str =~ s{%(..)}{chr(hex($1))}eg;
+            $Backups[$i]{share2path} = eval $str;
+            if ( $@ ) {
+                print(STDERR "BackupInfoRead: host $host: can't rebuild share2path from $str\n");
+            }
+        }
+        $Backups[$i]{comment} =~ s{%(..)}{chr(hex($1))}eg;
     }
     return @Backups;
 }
@@ -114,9 +124,19 @@ sub BackupInfoWrite
     #
     for ( $i = 0 ; $i < @Backups ; $i++ ) {
         my %b = %{$Backups[$i]};
+        if ( ref($b{share2path}) eq 'HASH' ) {
+            my $dump = Data::Dumper->new([$b{share2path}]);
+            $dump->Indent(0);
+            $dump->Sortkeys(1);
+            $dump->Terse(1);
+            my $value = $dump->Dump;
+            $value =~ s{([%\t\n\r])}{sprintf("%%%02x", ord($1))}eg;
+            $b{share2path} = $value;
+        }
+        $b{comment} =~ s{([%\t\n\r])}{sprintf("%%%02x", ord($1))}eg;
         $contents .= join("\t", @b{@{$s->{BackupFields}}}) . "\n";
     }
-    
+
     #
     # Write the file
     #
@@ -133,7 +153,7 @@ sub RestoreInfoRead
         $locked = 1;
     }
     if ( open($resFd, "$s->{TopDir}/pc/$host/restores") ) {
-	binmode($resFd);
+        binmode($resFd);
         while ( <$resFd> ) {
             s/[\n\r]+//;
             next if ( !/^(\d+.*)/ );
@@ -231,7 +251,7 @@ sub TextFileWrite
         return "TextFileWrite: can't create directory $dir" if ( $@ );
     }
     if ( open($fd, ">", "$file.new") ) {
-	binmode($fd);
+        binmode($fd);
         print $fd $contents;
         close($fd);
         #
@@ -246,17 +266,23 @@ sub TextFileWrite
             }
             close($fd);
         }
+        #
+        # use same gid as original file
+        #
+        my $uid = (stat("$file.new"))[4];
+        my $gid = (stat($file))[5];
+        chown($uid, $gid, "$file.new") if ( defined($uid) && defined($gid) );
     }
     if ( $fileOk ) {
         my($locked, $lockFd);
-        
+
         if ( open($lockFd, ">", "$dir/LOCK") ) {
             $locked = 1;
             flock($lockFd, LOCK_EX);
         }
         if ( -s "$file" ) {
-            unlink("$file.old")           if ( -f "$file.old" );
-            rename("$file", "$file.old")  if ( -f "$file" );
+            unlink("$file.old")          if ( -f "$file.old" );
+            rename("$file", "$file.old") if ( -f "$file" );
         } else {
             unlink("$file") if ( -f "$file" );
         }
@@ -280,9 +306,9 @@ sub ConfigPath
         return "$s->{ConfDir}/pc/$host.pl";
     } else {
         return "$s->{TopDir}/pc/$host/config.pl"
-            if ( -f "$s->{TopDir}/pc/$host/config.pl" );
+          if ( -f "$s->{TopDir}/pc/$host/config.pl" );
         return "$s->{ConfDir}/$host.pl"
-            if ( $host ne "config" && -f "$s->{ConfDir}/$host.pl" );
+          if ( $host ne "config" && -f "$s->{ConfDir}/$host.pl" );
         return "$s->{ConfDir}/pc/$host.pl";
     }
 }
@@ -295,14 +321,14 @@ sub ConfigDataRead
     #
     # TODO: add lock
     #
-    my $conf = $prevConfig || {};
+    my $conf       = $prevConfig || {};
     my $configPath = $s->ConfigPath($host);
 
     push(@configs, $configPath) if ( -f $configPath );
     foreach $config ( @configs ) {
         %Conf = %$conf;
         if ( !defined($ret = do $config) && ($! || $@) ) {
-            $mesg = "Couldn't open $config: $!" if ( $! );
+            $mesg = "Couldn't open $config: $!"    if ( $! );
             $mesg = "Couldn't execute $config: $@" if ( $@ );
             $mesg =~ s/[\n\r]+//;
             return ($mesg, $conf);
@@ -315,9 +341,9 @@ sub ConfigDataRead
     #
     foreach my $param ( qw(BackupFilesOnly BackupFilesExclude) ) {
         next if ( !defined($conf->{$param}) || ref($conf->{$param}) eq "HASH" );
-        $conf->{$param} = [ $conf->{$param} ]
-                                if ( ref($conf->{$param}) ne "ARRAY" );
-        $conf->{$param} = { "*" => $conf->{$param} };
+        $conf->{$param} = [$conf->{$param}]
+          if ( ref($conf->{$param}) ne "ARRAY" );
+        $conf->{$param} = {"*" => $conf->{$param}};
     }
 
     #
@@ -325,18 +351,38 @@ sub ConfigDataRead
     # BlackoutHourEnd, and BlackoutWeekDays parameters.
     #
     if ( defined($conf->{BlackoutHourBegin}) ) {
-        push(@{$conf->{BlackoutPeriods}},
-             {
-                 hourBegin => $conf->{BlackoutHourBegin},
-                 hourEnd   => $conf->{BlackoutHourEnd},
-                 weekDays  => $conf->{BlackoutWeekDays},
-             }
+        push(
+            @{$conf->{BlackoutPeriods}},
+            {
+                hourBegin => $conf->{BlackoutHourBegin},
+                hourEnd   => $conf->{BlackoutHourEnd},
+                weekDays  => $conf->{BlackoutWeekDays},
+            }
         );
         delete($conf->{BlackoutHourBegin});
         delete($conf->{BlackoutHourEnd});
         delete($conf->{BlackoutWeekDays});
     }
 
+    #
+    # Check that certain settings have valid values
+    #
+    if (   $conf->{BackupPCNightlyPeriod} != 1
+        && $conf->{BackupPCNightlyPeriod} != 2
+        && $conf->{BackupPCNightlyPeriod} != 4
+        && $conf->{BackupPCNightlyPeriod} != 8
+        && $conf->{BackupPCNightlyPeriod} != 16 ) {
+        $conf->{BackupPCNightlyPeriod} = 1;
+    }
+    if (   $conf->{PoolSizeNightlyUpdatePeriod} != 0
+        && $conf->{PoolSizeNightlyUpdatePeriod} != 1
+        && $conf->{PoolSizeNightlyUpdatePeriod} != 2
+        && $conf->{PoolSizeNightlyUpdatePeriod} != 4
+        && $conf->{PoolSizeNightlyUpdatePeriod} != 8
+        && $conf->{PoolSizeNightlyUpdatePeriod} != 16 ) {
+        $conf->{PoolSizeNightlyUpdatePeriod} = 16;
+    }
+
     return (undef, $conf);
 }
 
@@ -368,7 +414,7 @@ sub ConfigFileMerge
         # Match existing settings in current config file
         #
         open($configFd, $inFile)
-            || return ("ConfigFileMerge: can't open/read $inFile", undef);
+          || return ("ConfigFileMerge: can't open/read $inFile", undef);
         binmode($configFd);
 
         while ( <$configFd> ) {
@@ -376,7 +422,7 @@ sub ConfigFileMerge
                 my $var = $1;
                 $skipExpr = "\$fakeVar = $2\n";
                 if ( exists($newConf->{$var}) ) {
-                    my $d = Data::Dumper->new([$newConf->{$var}], [*value]);
+                    my $d = Data::Dumper->new([$newConf->{$var}]);
                     $d->Indent(1);
                     $d->Terse(1);
                     $d->Sortkeys(1);
@@ -390,9 +436,7 @@ sub ConfigFileMerge
             } else {
                 $contents .= $_;
             }
-            if ( defined($skipExpr)
-                    && ($skipExpr =~ /^\$fakeVar = *<</
-                        || $skipExpr =~ /;[\n\r]*$/) ) {
+            if ( defined($skipExpr) && ($skipExpr =~ /^\$fakeVar = *<</ || $skipExpr =~ /;[\n\r]*$/) ) {
                 #
                 # if we have a complete expression, then we are done
                 # skipping text from the original config file.
@@ -409,15 +453,15 @@ sub ConfigFileMerge
     # Add new entries not matched in current config file
     #
     foreach my $var ( sort(keys(%$newConf)) ) {
-	next if ( $done->{$var} );
-	my $d = Data::Dumper->new([$newConf->{$var}], [*value]);
-	$d->Indent(1);
-	$d->Terse(1);
+        next if ( $done->{$var} );
+        my $d = Data::Dumper->new([$newConf->{$var}]);
+        $d->Indent(1);
+        $d->Terse(1);
         $d->Sortkeys(1);
-	my $value = $d->Dump;
-	$value =~ s/(.*)\n/$1;\n/s;
-	$contents .= "\$Conf{$var} = " . $value;
-	$done->{$var} = 1;
+        my $value = $d->Dump;
+        $value =~ s/(.*)\n/$1;\n/s;
+        $contents .= "\$Conf{$var} = " . $value;
+        $done->{$var} = 1;
     }
     return (undef, $contents);
 }
@@ -438,9 +482,10 @@ sub StatusDataRead
 
     %Status = ();
     %Info   = ();
-    if ( -f "$s->{LogDir}/status.pl"
-            && !defined($ret = do "$s->{LogDir}/status.pl") && ($! || $@) ) {
-        $mesg = "Couldn't open $s->{LogDir}/status.pl: $!" if ( $! );
+    if (   -f "$s->{LogDir}/status.pl"
+        && !defined($ret = do "$s->{LogDir}/status.pl")
+        && ($! || $@) ) {
+        $mesg = "Couldn't open $s->{LogDir}/status.pl: $!"    if ( $! );
         $mesg = "Couldn't execute $s->{LogDir}/status.pl: $@" if ( $@ );
         $mesg =~ s/[\n\r]+//;
         rename("$s->{LogDir}/status.pl", "$s->{LogDir}/status.pl.bad");
@@ -453,9 +498,7 @@ sub StatusDataWrite
 {
     my($s, $status, $info) = @_;
 
-    my($dump) = Data::Dumper->new(
-                     [  $info, $status],
-                     [qw(*Info *Status)]);
+    my($dump) = Data::Dumper->new([$info, $status], [qw(*Info *Status)]);
     $dump->Indent(1);
     my $text = $dump->Dump;
     $s->TextFileWrite("$s->{LogDir}/status.pl", $text);
@@ -495,7 +538,7 @@ sub HostInfoRead
         #
         # Split on white space, except if preceded by \
         # using zero-width negative look-behind assertion
-	# (always wanted to use one of those).
+        # (always wanted to use one of those).
         #
         @fld = split(/(?<!\\)\s+/, $1);
         #
@@ -508,7 +551,7 @@ sub HostInfoRead
             if ( defined($host) ) {
                 next if ( lc($fld[0]) ne lc($host) );
                 @{$hosts{lc($fld[0])}}{@hdr} = @fld;
-		close($hostFd);
+                close($hostFd);
                 if ( $locked ) {
                     flock($lockFd, LOCK_UN);
                     close($lockFd);
@@ -546,9 +589,7 @@ sub HostInfoWrite
     }
     foreach my $host ( keys(%$hosts) ) {
         my $name = "$hosts->{$host}{host}";
-        my $rest = "\t$hosts->{$host}{dhcp}"
-                 . "\t$hosts->{$host}{user}"
-                 . "\t$hosts->{$host}{moreUsers}";
+        my $rest = "\t$hosts->{$host}{dhcp}\t$hosts->{$host}{user}\t$hosts->{$host}{moreUsers}";
         $name =~ s/ /\\ /g;
         $rest =~ s/ //g;
         $hostText->{$host} = $name . $rest;
